#include "mainwindow.h"
#include "ui_mainwindow.h"


/*
 *	###############################################################################################
 *  #                                        Constructors                                         #
 *	###############################################################################################
 */

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), ui(new Ui::MainWindow)
{
	ui->setupUi(this);
	getSettings();
}

MainWindow::~MainWindow()
{
	delete ui;
}

void MainWindow::closeEvent(QCloseEvent * event)
{
	saveSettings(true);
	event->accept();
}

/*
 *	###############################################################################################
 *  #                                      Ui Private Slots                                       #
 *	###############################################################################################
 */

void MainWindow::on_actionRun_triggered()
{
	run();
}

void MainWindow::on_actionSave_settings_triggered()
{
	setChanged();
	saveSettings(true);
}

void MainWindow::on_actionAbout_triggered()
{
	qApp->aboutQt();
}

void MainWindow::on_actionView_Help_triggered()
{
	AboutQtTwitGraph aboutWindow(this);
	aboutWindow.exec();
}

void MainWindow::on_run_pushButton_clicked()
{
	run();
}



/* ============ Tab1 - General settings ======================================================== */

void MainWindow::on_path_button_clicked()
{
	QVariantMap idpw;
	QString path = QFileDialog::getExistingDirectory(this);
	if(!path.isEmpty())	ui->path_lineEdit->setText(path);
}

/* ============ Tab2 - Datasift accounts ======================================================= */

void MainWindow::on_datasiftAdd_pushButton_clicked()
{
	addDatasiftIdpw(ui->usernameValue_lineEdit->text(), ui->apikeyValue_lineEdit->text());
}

void MainWindow::on_datasiftClear_pushButton_clicked()
{
	ui->usernameValue_lineEdit->setText("");
	ui->apikeyValue_lineEdit->setText("");
}

void MainWindow::on_datasiftDelete_pushButton_clicked()
{
	delete ui->datasiftIdpws_treeWidget->currentItem();
}

void MainWindow::datasiftDeleteFirst()
{
	delete ui->datasiftIdpws_treeWidget->topLevelItem(0);
}

void MainWindow::on_datasiftDeleteAll_pushButton_clicked()
{
	if (QMessageBox::question(this, "Delete All", "Are you sure ?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
	{
		int idpwCount = ui->datasiftIdpws_treeWidget->topLevelItemCount();
		for(int i=0; i<idpwCount; i++)
			datasiftDeleteFirst();
	}
}

/* ============ Tab3 - Twitter accounts ======================================================== */

void MainWindow::on_twitterAdd_pushButton_clicked()
{
	this->setDisabled(true);

	TokenFetcher *tokenfetcher = new TokenFetcher(this);
	QObject::connect(tokenfetcher, SIGNAL(gotAccess(QString, QString, QString)), this, SLOT(addTwitterIdpw(QString,QString,QString)));
	tokenfetcher->getAccess();
	delete tokenfetcher;

	this->setDisabled(false);
}

void MainWindow::on_twitterDelete_pushButton_clicked()
{
	delete ui->twitterIdpws_treeWidget->currentItem();
}

void MainWindow::on_twitterDeleteAll_pushButton_clicked()
{
	if (QMessageBox::question(this, "Delete All", "Are you sure ?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
	{
		int idpwCount = ui->twitterIdpws_treeWidget->topLevelItemCount();
		for(int i=0; i<idpwCount; i++)
			delete ui->twitterIdpws_treeWidget->topLevelItem(0);
	}
}

/* ============ Others ========================================================================= */

void MainWindow::setChanged(bool value)
{
	_settingsChanged = value;
	if(value == true)
		ui->statusbar->showMessage("Settings have changed", 3000);
}

/*
 *	###############################################################################################
 *  #                                        Private Slots                                        #
 *	###############################################################################################
 */

void MainWindow::addTwitterIdpw(QString username, QString token, QString tokenSecret, bool change)
{
	if(username != "" && token != "" && tokenSecret != "")
	{
		ui->twitterIdpws_treeWidget->addTopLevelItem(new QTreeWidgetItem(QStringList() << username << token << tokenSecret));
		if(change) setChanged();
	}
	else
		QMessageBox::critical(this, "Error", "Critical error getting twitter tokens");
}

/*
 *	###############################################################################################
 *  #                                        Public Slots                                         #
 *	###############################################################################################
 */







/*
 *	###############################################################################################
 *  #                                           Private                                           #
 *	###############################################################################################
 */

void MainWindow::run()
{
	Setting * settings = new Setting(this);
	settings->setRuntime(timeToMs(ui->runtimeTime_timeEdit->time(), ui->runtimeDay_spinBox->value()));
	settings->setNodetime(timeToMs(ui->nodeTime_timeEdit->time(), ui->nodeDay_spinBox->value()));
	settings->setEdgetime(timeToMs(ui->edgeTime_timeEdit->time(), ui->edgeDay_spinBox->value()));
	settings->streamType = ui->streamType_comboBox->currentIndex();
	settings->streamValue = ui->streamValue_textEdit->toPlainText();
	settings->path = ui->path_lineEdit->text();

	int datasiftIdpwCount = ui->datasiftIdpws_treeWidget->topLevelItemCount();
	for(int i=0; i<datasiftIdpwCount; i++)
		settings->datasift->addIdpw(ui->datasiftIdpws_treeWidget->topLevelItem(i)->text(0), ui->datasiftIdpws_treeWidget->topLevelItem(i)->text(1));

	QObject::connect(settings->datasift, SIGNAL(depleted()), this, SLOT(datasiftDeleteFirst()));

	int twitterIdpwCount = ui->twitterIdpws_treeWidget->topLevelItemCount();
	for(int i=0; i<twitterIdpwCount; i++)
		settings->twitter->addIdpw(ui->twitterIdpws_treeWidget->topLevelItem(i)->text(1), ui->twitterIdpws_treeWidget->topLevelItem(i)->text(2));



	RunWindow runWindow(this, settings); //ui->datasiftIdpws_treeWidget, ui->twitterIdpws_treeWidget);
	runWindow.exec();
}









void MainWindow::addDatasiftIdpw(QString username, QString apikey, bool change)
{
	if(username != "" && apikey != "")
	{
		ui->datasiftIdpws_treeWidget->addTopLevelItem(new QTreeWidgetItem(QStringList() << username << apikey));
		on_datasiftClear_pushButton_clicked();
		if(change) setChanged();
	}
	else
		QMessageBox::information(this,"Can't add datasift account","Username/Api Key is empty");
}

void MainWindow::getSettings()
{
	ui->statusbar->showMessage("Getting last settings");
	setChanged(false);

	bool error = false;
	QFile settingFile(SETTING_FILE);
	if (settingFile.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		QJson::Parser parser;
		bool parsed;
		QVariantMap settings = parser.parse(&settingFile,&parsed).toMap();
		if(parsed)
		{
			ui->runtimeDay_spinBox->setValue(settings["runtime"].toMap()["days"].toInt());
			ui->runtimeTime_timeEdit->setTime(settings["runtime"].toMap()["time"].toTime());

			ui->nodeDay_spinBox->setValue(settings["node"].toMap()["days"].toInt());
			ui->nodeTime_timeEdit->setTime(settings["node"].toMap()["time"].toTime());

			ui->edgeDay_spinBox->setValue(settings["edge"].toMap()["days"].toInt());
			ui->edgeTime_timeEdit->setTime(settings["edge"].toMap()["time"].toTime());

			if((settings["stream"].toMap()["type"].toString()=="hash"))
				ui->streamType_comboBox->setCurrentIndex(1);
			ui->streamValue_textEdit->setText(settings["stream"].toMap()["value"].toString());

			if(settings["path"].toString() == "")
				error = true;
			else
				ui->path_lineEdit->setText(settings["path"].toString());

			foreach (QVariant datasiftIdpw, settings["datasiftIdpw"].toList())
				addDatasiftIdpw(datasiftIdpw.toMap()["username"].toString(), datasiftIdpw.toMap()["apikey"].toString(), false);
			foreach (QVariant twitterIdpw, settings["twitterIdpw"].toList())
				addTwitterIdpw(twitterIdpw.toMap()["username"].toString(), twitterIdpw.toMap()["token"].toString(), twitterIdpw.toMap()["tokenSecret"].toString(), false);
		}
		else
			error = true;
		settingFile.close();
	}
	else
		error = true;

	if(error)
	{
		setChanged();
		ui->path_lineEdit->setText(QDir::currentPath());
		ui->statusbar->showMessage("Could not read the setting file, all settings where reseted", 5000);
	}
	else
	{
		ui->statusbar->showMessage("Last settings successfully imported", 5000);
	}

	QObject::connect(ui->runtimeTime_timeEdit, SIGNAL(timeChanged(QTime)), this, SLOT(setChanged()));
	QObject::connect(ui->runtimeDay_spinBox, SIGNAL(valueChanged(int)), this, SLOT(setChanged()));
	QObject::connect(ui->nodeTime_timeEdit, SIGNAL(timeChanged(QTime)), this, SLOT(setChanged()));
	QObject::connect(ui->nodeDay_spinBox, SIGNAL(valueChanged(int)), this, SLOT(setChanged()));
	QObject::connect(ui->edgeTime_timeEdit, SIGNAL(timeChanged(QTime)), this, SLOT(setChanged()));
	QObject::connect(ui->edgeDay_spinBox, SIGNAL(valueChanged(int)), this, SLOT(setChanged()));
	QObject::connect(ui->streamType_comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setChanged()));
	QObject::connect(ui->streamValue_textEdit, SIGNAL(textChanged()), this, SLOT(setChanged()));
	QObject::connect(ui->path_lineEdit,SIGNAL(textChanged(QString)),this,SLOT(setChanged()));
}

void MainWindow::saveSettings(bool notification)
{
	if(_settingsChanged)
	{
		QVariantMap settings;
			QVariantMap runtime;
				runtime.insert("days", QString::number(ui->runtimeDay_spinBox->value()));
				runtime.insert("time", ui->runtimeTime_timeEdit->time().toString());
			settings.insert("runtime", runtime);
			QVariantMap node;
				node.insert("days", QString::number(ui->nodeDay_spinBox->value()));
				node.insert("time", ui->nodeTime_timeEdit->time().toString());
			settings.insert("node", node);
			QVariantMap edge;
				edge.insert("days", QString::number(ui->edgeDay_spinBox->value()));
				edge.insert("time", ui->edgeTime_timeEdit->time().toString());
			settings.insert("edge", edge);
			settings.insert("path", ui->path_lineEdit->text());
			QVariantMap stream;
				switch (ui->streamType_comboBox->currentIndex())
				{
					case 0:
					{
						stream.insert("type", "csdl");
						break;
					}
					case 1:
					{
						stream.insert("type", "hash");
						break;
					}
				}
				stream.insert("value", ui->streamValue_textEdit->toPlainText());
			settings.insert("stream", stream);

			QVariantList datasiftIdpw;
				int datasiftIdpwCount = ui->datasiftIdpws_treeWidget->topLevelItemCount();
				for(int i=0; i<datasiftIdpwCount; i++)
				{
					QVariantMap idpw;
						idpw.insert("username", ui->datasiftIdpws_treeWidget->topLevelItem(i)->text(0));
						idpw.insert("apikey", ui->datasiftIdpws_treeWidget->topLevelItem(i)->text(1));
					datasiftIdpw.append(idpw);
				}
			settings.insert("datasiftIdpw", datasiftIdpw);

			QVariantList twitterIdpw;
				int twitterIdpwCount = ui->twitterIdpws_treeWidget->topLevelItemCount();
				for(int i=0; i<twitterIdpwCount; i++)
				{
					QVariantMap idpw;
						idpw.insert("username", ui->twitterIdpws_treeWidget->topLevelItem(i)->text(0));
						idpw.insert("token", ui->twitterIdpws_treeWidget->topLevelItem(i)->text(1));
						idpw.insert("tokenSecret", ui->twitterIdpws_treeWidget->topLevelItem(i)->text(2));
					twitterIdpw.append(idpw);
				}
			settings.insert("twitterIdpw", twitterIdpw);

		QJson::Serializer serializer;
		QByteArray json = serializer.serialize(settings);
		QFile settingFile(SETTING_FILE);
		if(settingFile.open(QIODevice::WriteOnly | QIODevice::Text))
		{
			if(settingFile.write(json))
			{
				setChanged(false);
				if(notification)
					QMessageBox::information(this,"Settings saved","New settings where successfully saved");
			}
			settingFile.close();
		}
	}
}

