#include "runwindow.h"
#include "ui_runwindow.h"

RunWindow::RunWindow(Setting *settings)
{
	RunWindow(0, settings);
}


RunWindow::RunWindow(QWidget *parent, Setting *settings) : QDialog(parent),
															ui(new Ui::RunWindow),
															settings(settings)
{
	ui->setupUi(this);
	settings->setLog(ui->display_textBrowser);

	launch();
}

RunWindow::~RunWindow()
{
	delete settings;
	delete ui;
}

void RunWindow::on_runningCancel_pushButton_clicked()
{
	stop();
}

void RunWindow::launch()
{
	ui->runningBar_progressBar->setValue(0);

	timer = new TimerManager(this);
	graph = new Graph(this);
	apidatasift = new ApiDatasift(this, &graph, settings);

	switch(settings->streamType)
	{
		case 0:
		{
			apidatasift->setCsdl(settings->streamValue);
			break;
		}
		case 1:
		{
			apidatasift->setHash(settings->streamValue);
			break;
		}
	}

	apitwitter = new ApiTwitter(this, &graph, settings);

	timer->addTimer(settings->nodetime(), apidatasift, SLOT(refresh()));
	timer->addTimer(settings->edgetime(), apitwitter, SLOT(run()));
	timer->addTimer(settings->runtime()/100, this, SLOT(updateBar()));
	timer->addTimer(100, this, SLOT(updateInfo()));
	timer->addTimer(settings->runtime(), this, SLOT(stop()), true);

	QObject::connect(apidatasift, SIGNAL(getFriends(int)), apitwitter, SLOT(updateFollowers(int)));

	settings->running = true;
	timer->start();
}

void RunWindow::stop()
{
//	Wait waitDialog(this);
//	waitDialog.show();

	timer->stop();
	settings->running = false;

	delete apidatasift;
	delete apitwitter;

	graph->makeXml(settings->path);

	delete timer;
	delete graph;

//	waitDialog.hide();

	close();
}

void RunWindow::updateBar()
{
	ui->runningBar_progressBar->setValue(ui->runningBar_progressBar->value() + 1);
}

void RunWindow::updateInfo()
{
	ui->tweets_label->setText(QString::number(apidatasift->tweetNumber())+" tweets");
	ui->node_label->setText(QString::number(graph->nodesNumber())+" tweeters");
	ui->edge_label->setText(QString::number(graph->edgesNumber())+" links");
}
